@@ -8,10 +8,54 @@ mod mutvec;
 mod vecvecf64;
 pub mod functions;
 
-use crate::functions::GI; 
 /// simple error handling
 use anyhow::{Result,bail}; 
 
+/// macro here!() gives &str with the current file:line path::function for error messages
+#[macro_export]
+macro_rules! here {
+    () => {{
+        fn f() {}
+        fn type_name_of<T>(_: T) -> &'static str {
+            std::any::type_name::<T>()
+        }
+        let name = type_name_of(f);
+        // For function name only:
+        // let fnct = match &name[..name.len()-3].rfind(':') {
+        //    Some(pos) => &name[pos + 1..name.len() - 3],
+        //    None => &name[..name.len()-3],
+        // };
+        format!("\n{}:{} {}", file!(), line!(), &name[..name.len()-3])
+    }}
+}
+
+/// GreenIt (GI) struct facilitates printing (in green) any
+/// singular type that has Display implemented.
+pub struct GI<T: std::fmt::Display>(pub T);
+impl<T: std::fmt::Display> std::fmt::Display for GI<T> {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        write!(f, "\x1B[01;92m{}\x1B[0m", self.0.to_string())
+    }
+}
+
+/// GreenVec (GV) struct facilitates printing (in green) vector
+/// of any end type that has Display implemented.
+pub struct GV<'a, T: std::fmt::Display>(pub &'a[T]);
+impl<'a, T: std::fmt::Display> std::fmt::Display for GV<'a,T> {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        let mut s = String::from("\x1B[01;92m[");
+        let n = self.0.len();
+        if n > 0 {
+            s.push_str(&self.0[0].to_string()); // first item
+            for i in 1..n {
+                s.push_str(", ");
+                s.push_str(&self.0[i].to_string());
+            }
+        }
+        write!(f, "{}]\x1B[0m", s)
+    }
+}
+
 /// Median and quartiles
 #[derive(Default)]
 pub struct Med {
@@ -138,6 +182,8 @@ pub trait Vecf64 {
     fn autocorr(self) -> f64;
     /// Lower triangular part of a covariance matrix for a single f64 vector.
     fn covone(self, m:&[f64]) -> Vec<f64>;
+    /// Reconstructs the full symmetric matrix from its lower diagonal compact form
+    fn symmatrix(self) -> Vec<Vec<f64>>;
  
     /// Minimum, minimum's index, maximum, maximum's index.
     fn minmax(self) -> (f64, usize, f64, usize); 
@@ -158,7 +204,7 @@ pub trait Vecf64 {
     fn sortm(self, ascending:bool) -> Vec<f64>;
 
     /// Ranking with only n*log(n) complexity, using 'mergesort'
-    fn mergerank(self) -> Vec<usize>;
+    fn rank(self, ascending:bool) -> Vec<usize>;
     /// Immutable merge sort, makes a sort index
     fn mergesort(self, i:usize, n:usize) -> Vec<usize>;
 }
@@ -256,6 +302,7 @@ pub trait VecVecu8 {
     fn wgmedian(self, ws:&[u8], eps: f64) -> Vec<f64>;
     /// Lower triangular part of a covariance matrix of a Vec of u8 vectors.
     fn covar(self, med:&[f64]) -> Vec<f64>; 
+    fn wcovar(self, ws:&[u8], m:&[f64]) -> Vec<f64>;
 }
 
 /// Methods applicable to vector of vectors of <f64>
@@ -325,7 +372,7 @@ pub trait VecVecf64 {
 /// Methods to manipulate indices of Vec<usize> type
 pub trait Indices {
     /// Reverse index
-    fn revindex(self) -> Vec<usize>;
+    fn invindex(self) -> Vec<usize>;
     /// Collects f64 values from `v` as per indices in self.
     fn unindex(self, ascending:bool, v:&[f64]) -> Vec<f64>;
     /// Collects u8 from `v` as per indices in self.
