@@ -1,4 +1,4 @@
-use crate::{MutVectors, Vecf64, Indices, functions::emsg};
+use crate::{MutVectors, Vecf64, Indices, here};
 
 impl Vecf64 for &[f64] {
 
@@ -197,8 +197,8 @@ impl Vecf64 for &[f64] {
     /// assert_eq!(v1.spearmancorr(&v2),-0.1076923076923077);
     /// ```
     fn spearmancorr(self, v: &[f64]) -> f64 {
-        let xvec = self.mergerank();
-        let yvec = v.mergerank();
+        let xvec = self.rank(true);
+        let yvec = v.rank(true);
         // It is just Pearson's correlation of ranks
         xvec.ucorrelation(&yvec)
     }
@@ -214,7 +214,7 @@ impl Vecf64 for &[f64] {
     /// assert_eq!(v1.kazutsugi(),0.3);
     /// ```
     fn kazutsugi(self) -> f64 {
-        let xvec = self.mergerank();
+        let xvec = self.rank(true);
         let yvec:Vec<f64> = vec![4.,3.,2.,1.,0.];
         let (mut sxy, mut sx2) = (0_f64,0_f64);
         const MY:f64 = 2.;  // y mean 
@@ -249,7 +249,7 @@ impl Vecf64 for &[f64] {
         let nf = self.len() as f64;
         (sxy - sx / nf * sy) / ((sx2 - sx / nf * sx) * (sy2 - sy / nf * sy)).sqrt()
     }
-
+ 
     /// Finds minimum, minimum's index, maximum, maximum's index of &[f64]
     /// Here self is usually some data, rather than a vector
     fn minmax(self) -> (f64, usize, f64, usize) {
@@ -284,7 +284,7 @@ impl Vecf64 for &[f64] {
     /// This then is the right index to use for looking up cummulative probability density functions. 
     fn binsearch(self, v: f64) -> usize {
         let n = self.len();
-        if n < 2 { emsg(file!(), line!(), "binsearch list is too short!"); }
+        if n < 2 { panic!("{} list is too short!",here!()) }
         if v < self[0] { return 0_usize }; // v is smaller than the first item
         let mut hi = n-1; // valid index of the last item
         if v > self[hi] { return n }; // indicates that v is greater than the last item
@@ -395,12 +395,23 @@ impl Vecf64 for &[f64] {
 
     /// Ranking of self by inverting the (merge) sort index.  
     /// Sort index is in sorted order, giving indices to the original data positions.
-    /// Ranking is in  original data order, giving their positions in the sort index.
-    /// Thus they are in an inverse relationship, easily converted by `.revindex()`
-    /// Very fast ranking of many f64 items, ranking `self` with only n*(log(n)+1) complexity.
-    fn mergerank(self) -> Vec<usize> {
-        let indx = self.mergesort(0,self.len());
-        indx.revindex()    
+    /// Ranking is in  original data order, giving their positions in the sorted order (sort index).
+    /// Thus they are in an inverse relationship, easily converted by `.invindex()`
+    /// Fast ranking of many f64 items, ranking `self` with only n*(log(n)+1) complexity.
+    fn rank(self, ascending:bool) -> Vec<usize> {
+        let n = self.len();
+        let sortindex = self.mergesort(0,n);
+        let mut rankvec:Vec<usize> = vec![0;n];
+        if ascending { 
+            for (i,&sortpos) in sortindex.iter().enumerate() {
+                rankvec[sortpos] = i
+            } 
+        } else { // rank in the order of descending values
+            for (i,&sortpos) in sortindex.iter().enumerate() {
+                    rankvec[sortpos] = n-i-1 
+            }
+        }
+        rankvec 
     }    
 
     /// Doubly recursive non-destructive merge sort. The data is read-only, it is not moved or mutated. 
@@ -447,5 +458,24 @@ impl Vecf64 for &[f64] {
         }
         cov
     }
+
+    /// Reconstructs the full symmetric square matrix from its lower diagonal compact form,
+    /// as produced by covar, covone, wcovar
+    fn symmatrix(self) -> Vec<Vec<f64>> {
+        // solve quadratic equation to find the dimension of the square matrix
+        let n = (((8*self.len()+1) as f64).sqrt() as usize - 1)/2;
+        let mut mat = vec![vec![0_f64;n];n]; // create the square matrix 
+        let mut selfindex = 0;
+        for row in 0..n {     
+            for column in 0..row { // excludes the diagonal  
+                mat[row][column] = self[selfindex]; // just copy the value into the lower triangle
+                mat[column][row] = self[selfindex]; // and into transposed upper position 
+                selfindex += 1 // move to the next input value
+            } // this row of lower triangle finished
+            mat[row][row] = self[selfindex];  // now the diagonal element, no transpose
+            selfindex += 1 // move to the next input value
+        }
+        mat
+    }
     
 }
