@@ -2,10 +2,11 @@
 #![allow(dead_code)]
 #[cfg(test)]
 
-use anyhow::{Result};
-use rstats::{Stats,MutVectors,Vecf64,VecVecf64,Vecu8,VecVecu8,Indices};
-use rstats::functions::{GI,GV,genvec,genvecu8};
 use devtimer::DevTime;
+use anyhow::{Result};
+
+use rstats::{Stats,MutVectors,Vecf64,VecVecf64,Vecu8,VecVecu8,Indices,GI,GV};
+use rstats::functions::{genvec,genvecu8};
 
 pub const EPS:f64 = 1e-7;
 #[test]
@@ -24,8 +25,11 @@ fn u8() -> Result<()> {
    let n = 7_usize;
    println!("Testing on a random set of {} points in {} d space\n",GI(n),GI(d));
    let pt = genvecu8(d,n,5,7); // random test data 
-   println!("Covariances:\n{}",GV(pt.covar(&pt.acentroid())));
-   println!("Comediances:\n{}\n",GV(pt.covar(&pt.gmedian(EPS))));
+   let cov = pt.covar(&pt.acentroid());
+   let com = pt.covar(&pt.gmedian(EPS));
+   println!("Covariances:\n{}",GV(&cov));
+   println!("Comediances:\n{}",GV(&com));
+   println!("Cityblock Distance: {}\n",GI(cov.cityblockd(&com)));
    Ok(())
 }
 
@@ -34,7 +38,7 @@ fn fstats() -> Result<()> {
    let v0 = vec![1_u8,2,3,4,5,6,7,8,9,10,11,12,13,14,15];
    let v1 = v0.vecu8asvecf64(); // testing the cast
    println!("\n{:?}",v1);
-   println!("Linear transform:\n{}",GV(v1.lintrans()));
+   println!("Linear transform:\n{}",GV(&v1.lintrans()));
    println!("Arithmetic mean:{}",GI(v1.amean().unwrap()));
    println!("Geometric mean:\t{}",GI(v1.gmean().unwrap()));
    println!("Harmonic mean:\t{}",GI(v1.hmean().unwrap()));
@@ -60,32 +64,43 @@ fn intstats() -> Result<()> {
 #[test]
 fn vecf64() -> Result<()> { 
    let v1 = vec![1_f64,2.,3.,4.,5.,6.,7.,8.,9.,10.,11.,12.,13.,14.,15.];
-   println!("\n{:?}",v1);
+   println!("\n{}",GV(&v1));
    let v2 = vec![1_f64,14.,2.,13.,3.,12.,4.,11.,5.,10.,6.,9.,7.,8.,15.];
-   println!("{:?}",v2);
+   println!("{}",GV(&v2));
+   println!("Sorted:       {}",GV(&v2.sortm(true)));
+   println!("Sorted:       {}",GV(&v2.mergesort(0,v2.len()).unindex(true,&v2))); 
+   println!("Sorted:       {}",GV(&v2.rank(false).invindex().unindex(false,&v2)));   
+   println!("Ranks:        {}",GV(&v2.rank(true))); 
+   println!("Ranks rev:    {}",GV(&v2.rank(true).iter().rev().collect::<Vec<&usize>>())); 
+   println!("Ranks desc:   {}",GV(&v2.rank(false))); // not the same as ranks reversed!!   
+   println!("Sort index:   {}",GV(&v2.mergesort(0,v2.len()))); 
+   println!("Ix from Ranks:{}",GV(&v2.rank(true).invindex()));
+   println!("Sort ix desc: {}",GV(&v2.rank(false).invindex()));
+   println!("Ranks from ix:{}",GV(&v2.rank(false).invindex().invindex()));
+   println!("Sorted rev:   {}",GV(&v2.sortm(false)));
+   println!("Sorted rev:   {}",GV(&v2.sortm(true).iter().rev().collect::<Vec<&f64>>()));
+   println!("Sorted rev:   {}",GV(&v2.mergesort(0,v2.len()).unindex(false,&v2)));   
+   println!("Sorted rev:   {}",GV(&v2.rank(false).invindex().unindex(true,&v2))); 
+
    let (v,vi) = v1.merge_immutable(&v1.mergesort(0,v1.len()), &v2, &v2.mergesort(0,v2.len()));
-   println!("Concatenated and Sorted:\n{}",GV(vi.unindex(true,&v))); 
-   println!("Rank:      {}",GV(v2.ranks().unwrap()));
-   println!("Sort index:{}",GV(v2.mergesort(0,v2.len())));
-   println!("R reversed:{}",GV(v2.mergerank().revindex()));    
-   println!("Mergerank: {}",GV(v2.mergerank()));
-   println!("Mrg.reverse sorted:{}",GV(v2.sortm(false)));
-   println!("Sorted:    {}",GV(v2.sortf()));
+   println!("Concatenated and Sorted:\n{}\n",GV(&vi.unindex(true,&v))); 
+ 
    println!("Pearson's Correlation:\t{}",GI(v1.correlation(&v2))); 
    println!("Kendall's Correlation:\t{}",GI(v1.kendalcorr(&v2)));  
    println!("Spearman's Correlation:\t{}",GI(v1.spearmancorr(&v2)));  
    println!("Cosine:\t\t\t{}",GI(v1.cosine(&v2))); 
-   println!("Cosine of ranks:\t{}",
-        GI(v1.ranks().unwrap().cosine(&v2.ranks().unwrap())));        
+   println!("Cosine of ranks:\t{}",GI(v1.ranks().unwrap().cosine(&v2.ranks().unwrap())));        
    println!("Euclidian distance:\t{}",GI(v1.vdist(&v2)));
-   println!("Difference magnitude:\t{}",GI(v1.vsub(&v2).as_slice().vmag()));   
-   println!("Vector difference:\n{}",GV(v1.vsub(&v2))); 
-   println!("Vector addition:\n{}",GV(v1.vadd(&v2)));  
+   println!("Difference magnitude:\t{}",GI(v1.vsub(&v2).vmag()));   
+   println!("Vector difference{}",GV(&v1.vsub(&v2))); 
+   println!("Vector sum:{}",GV(&v1.vadd(&v2)));  
    println!("Scalar product:\t\t{}",GI(v1.dotp(&v2)));
    println!("Parallelogram area:\t{}",GI(v1.varea(&v2))); 
    println!("Similarity:\t\t{}",GI(v1.vsim(&v2)));
    println!("Dissimilarity:\t\t{}\n",GI(v1.vdisim(&v2))); 
    // println!("Arc area:\t\t{}\n",GI(v1.varc(&v2)));
+   let sm = v1.symmatrix();
+   for i in 0..5 { eprintln!("{}",GV(&sm[i])) };  
    Ok(())
 }
 #[test]
@@ -128,7 +143,7 @@ fn vecvec() -> Result<()> {
    println!("Eccentricities\t{}",mu);  
    println!("Eccentricities\t{}",eccmed);
    let (_, seccs) = pt.sortedeccs(true,EPS); 
-   println!("Sorted eccs: {:?}\n", seccs);
+   println!("Sorted eccs: {}\n", GV(&seccs));
    let medcnt = seccs.binsearch(eccmed.median);
    println!("Items smaller or equal to median of eccs: {} last value: {}", GI(medcnt), GI(seccs[medcnt-1]));
    let mut weights = Vec::new();
@@ -146,7 +161,7 @@ fn trend() -> Result<()> {
    let d = 7_usize;
    let pt1 = genvec(d,28,13,19); // random test data 
    let pt2 = genvec(d,38,23,31);
-   println!("\nTrend vector:\n{}\n",GV(pt1.trend(EPS,pt2)));
+   println!("\nTrend vector:\n{}\n",GV(&pt1.trend(EPS,pt2)));
    Ok(())
 }
 
